import { getSigner, ndk } from '@/ndk';
import { NDKEvent } from '@nostr-dev-kit/ndk';
import { LitElement, css, html } from 'lit';
import { customElement, property, state } from 'lit/decorators.js';

@customElement('arx-arbor-post-creator')
export class ArborPostCreator extends LitElement {
  @property({ type: String })
  topicId = '';

  @state()
  private postContent = '';

  @state()
  private topic: NDKEvent | null = null;

  @state()
  private isCreating = false;

  @state()
  private error: string | null = null;

  static override styles = css`
    :host {
      display: block;
    }

    .container {
      display: flex;
      flex-direction: column;
      gap: 1rem;
    }

    .topic-id {
      color: #666;
      font-family: monospace;
    }

    textarea {
      width: 100%;
      min-height: 200px;
      padding: 0.75rem;
      border: 1px solid #ccc;
      border-radius: 0.5rem;
      resize: vertical;
      font-family: inherit;
      line-height: 1.5;
    }

    textarea:focus {
      outline: none;
      border-color: var(--primary-color, #3b82f6);
      box-shadow: 0 0 0 2px rgba(59, 130, 246, 0.1);
    }

    .error {
      color: #dc2626;
      font-size: 0.875rem;
    }

    .actions {
      display: flex;
      gap: 1rem;
      justify-content: flex-end;
    }

    .disabled {
      opacity: 0.5;
      cursor: not-allowed;
    }
  `;

  override connectedCallback() {
    super.connectedCallback();
    this.loadTopic();
  }

  private async loadTopic() {
    try {
      await getSigner();
      this.topic = await ndk.fetchEvent(this.topicId);
      if (!this.topic) {
        throw new Error('Could not load topic');
      }
    } catch (error) {
      console.error('Failed to load topic:', error);
    }
  }

  private async doCreatePost() {
    if (this.isCreating) return;
    if (!this.topic) return;

    if (this.postContent.length < 10) {
      this.error = 'Post content must be at least 10 characters long';
      return;
    }

    this.error = null;
    this.isCreating = true;

    const event = new NDKEvent(ndk);
    event.kind = 893;
    event.tags = [
      ['A', this.topic.tags.find((tag) => tag[0] === 'd')?.[1] || ''],
      ['E', this.topicId],
      ['p', this.topic.pubkey],
    ];
    event.content = this.postContent;

    try {
      await event.sign();
      await event.publish();
      // Reset form
      this.postContent = '';
    } catch (error) {
      console.error('Failed to create post:', error);
      this.error = 'Failed to create post. Please try again.';
    } finally {
      this.isCreating = false;
    }
  }

  private handleContentInput(e: InputEvent) {
    this.postContent = (e.target as HTMLTextAreaElement).value;
    if (this.error && this.postContent.length >= 10) {
      this.error = null;
    }
  }

  override render() {
    return html`
      <div class="container">
        <div class="topic-id">Topic ID: ${this.topicId}</div>

        <textarea
          placeholder="Post. You can use Markdown here."
          .value=${this.postContent}
          @input=${this.handleContentInput}
          ?disabled=${this.isCreating}
        ></textarea>

        ${this.error ? html` <div class="error">${this.error}</div> ` : null}

        <div class="actions">
          <arx-arbor-button
            @click=${() => this.dispatchEvent(new CustomEvent('cancel'))}
            ?disabled=${this.isCreating}
          >
            Cancel
          </arx-arbor-button>

          <arx-arbor-button
            @click=${this.doCreatePost}
            ?disabled=${this.isCreating}
          >
            ${this.isCreating ? 'Creating...' : 'Create'}
          </arx-arbor-button>
        </div>
      </div>
    `;
  }
}
