import { getSigner, ndk } from '@/ndk';
import type { NDKKind, NDKSubscription } from '@nostr-dev-kit/ndk';
import { LitElement, css, html } from 'lit';
import { customElement, property, state } from 'lit/decorators.js';

import '@components/Breadcrumbs';
import '@components/Arbor/ForumPost';
import '@components/Arbor/Button';

import { map } from 'lit/directives/map.js';
import { when } from 'lit/directives/when.js';

interface ForumPost {
  id: string;
  npub: string;
  date: Date;
  content: string;
}

@customElement('arx-arbor-topic-view')
export class ArborTopicView extends LitElement {
  @property({ type: String })
  topicId = '';

  @state()
  override title = '';

  @state()
  private posts: ForumPost[] = [];

  private subscription: NDKSubscription | undefined;

  static override styles = css`
    :host {
      display: block;
      margin: 0 auto;
      padding: 1rem;
      color: #1a1a1a;
      font-family: system-ui, -apple-system, BlinkMacSystemFont, sans-serif;
    }

    .topic-container {
      background: #ffffff;
      border-radius: 12px;
      box-shadow: 0 8px 24px rgba(0, 0, 0, 0.08);
      overflow: hidden;
      margin-top: 1.5rem;
      border: 1px solid rgba(0, 0, 0, 0.08);
    }

    .header {
      background: var(--primary);
      color: oklch(100% 0 0);
      padding: 1.5rem 2rem;
      font-weight: 600;
      font-size: 1.25rem;
      display: flex;
      justify-content: space-between;
      align-items: center;
      position: relative;
      text-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
      letter-spacing: 0.01em;
    }

    .posts-container {
      display: flex;
      flex-direction: column;
      gap: 0;
      background: #f9f9f9;
      padding: 1.5rem;
      min-height: 300px;
    }

    .actions {
      background: #f9f9f9;
      padding: 1.25rem 1.5rem;
      display: flex;
      justify-content: flex-end;
      border-top: 1px solid rgba(0, 0, 0, 0.06);
    }

    .empty-state {
      padding: 3rem;
      text-align: center;
      color: #666;
      font-style: italic;
    }

    @media (max-width: 768px) {
      .header {
        padding: 1.25rem 1.5rem;
        font-size: 1.1rem;
      }

      .posts-container {
        padding: 1rem;
      }

      .actions {
        padding: 1rem;
      }
    }

    @media (max-width: 480px) {
      :host {
        padding: 0.5rem;
      }

      .header {
        padding: 1rem;
      }
    }
  `;

  override async connectedCallback() {
    super.connectedCallback();
    await this.loadTopic();
  }

  override disconnectedCallback() {
    super.disconnectedCallback();
    if (this.subscription) {
      this.subscription.stop();
    }
  }

  private async loadTopic() {
    try {
      await getSigner();
      const event = await ndk.fetchEvent(this.topicId);

      if (!event) {
        throw new Error('Could not load topic');
      }

      this.title = event.tags.find((tag) => tag[0] === 'name')?.[1] || '';

      this.posts = [
        {
          id: event.id,
          npub: event.pubkey,
          date: new Date((event.created_at || 0) * 1000),
          content: event.content,
        },
      ];

      // Subscribe to new posts
      this.subscription = ndk
        .subscribe({
          kinds: [893 as NDKKind],
          '#E': [this.topicId],
        })
        .on('event', (event) => {
          this.posts = [
            ...this.posts,
            {
              id: event.id,
              npub: event.pubkey,
              date: new Date((event.created_at || 0) * 1000),
              content: event.content,
            },
          ];
        });
    } catch (error) {
      console.error('Failed to load topic:', error);
      alert('Could not load topic');
    }
  }

  override render() {
    const breadcrumbItems = [{ text: 'Home', href: '/' }, { text: 'Arbor', href: '/arbor' }, { text: this.title }];

    return html`
      <arx-breadcrumbs .items=${breadcrumbItems}></arx-breadcrumbs>

      <div class="topic-container">
        <div class="header">
          <span>${this.title || 'Loading topic...'}</span>
        </div>

        <div class="posts-container">
          ${when(
            this.posts.length === 0,
            () => html`<div class="empty-state">No posts available</div>`,
            () =>
              map(
                this.posts,
                (post) => html`
                  <arx-forum-post
                    id=${post.id}
                    .topicId=${this.topicId}
                    .npub=${post.npub}
                    .date=${post.date}
                    .content=${post.content}
                  ></arx-forum-post>
                `,
              ),
          )}
        </div>

        <div class="actions">
          <arx-arbor-button href="/arbor/new-post/${this.topicId}">
            New Post
          </arx-arbor-button>
        </div>
      </div>
    `;
  }
}
